# -*- coding: utf-8 -*-
"""
Created on Thu Jun 10 11:14:08 2021

@author: Vishwesh
"""
import numpy as np
import matplotlib.pyplot as plt
from Logic_Py.basic_gates import AND, OR, NOT, NAND, NOR,XNOR,XOR
#%%
def AND_AND (A,B,C,D):
    """realisation of AND-AND gate
    
    Parameters
    ----------
    A : list[]
    B : list[]
    C : list[]
    D : list[]

    Returns
    -------
    x : list[]
        intermediate output of A AND B
    y : list[]
        intermediate output of C AND D
    z : list[]
        Final output of x AND y

    """
    z=[]
    x=AND(A,B)
    y=AND(C,D)
    z=AND(x,y)
    return x,y,z
#%%
def AND_OR (A,B,C,D):
    """realisation of AND-OR gate
    
    Parameters
    ----------
    A : list[]
    B : list[]
    C : list[]
    D : list[]

    Returns
    -------
    x : list[]
        intermediate output of A AND B
    y : list[]
        intermediate output of C AND D
    z : list[]
        Final output of x OR y

    """
    z=[]
    x=AND(A,B)
    y=AND(C,D)
    z=OR(x,y)
    return x,y,z
#%%
def AND_NAND (A,B,C,D):
    """realisation of AND-NAND gate
    
    Parameters
    ----------
    A : list[]
    B : list[]
    C : list[]
    D : list[]

    Returns
    -------
    x : list[]
        intermediate output of A AND B
    y : list[]
        intermediate output of C AND D
    z : list[]
        Final output of x AND y

    """
    z=[]
    x=AND(A,B)
    y=AND(C,D)
    z=NAND(x,y)
    return x,y,z
#%%
def AND_NOR (A,B,C,D):
    """realisation of AND-NOR gate
    
    Parameters
    ----------
    A : list[]
    B : list[]
    C : list[]
    D : list[]

    Returns
    -------
    x : list[]
        intermediate output of A AND B
    y : list[]
        intermediate output of C AND D
    z : list[]
        Final output of x NOR y

    """
    z=[]
    x=AND(A,B)
    y=AND(C,D)
    z=NOR(x,y)
    return x,y,z
#%%
def OR_AND (A,B,C,D):
    """realisation of OR-AND gate
    
    Parameters
    ----------
    A : list[]
    B : list[]
    C : list[]
    D : list[]

    Returns
    -------
    x : list[]
        intermediate output of A OR B
    y : list[]
        intermediate output of C OR D
    z : list[]
        Final output of x AND y

    """
    z=[]
    x=OR(A,B)
    y=OR(C,D)
    z=AND(x,y)
    return x,y,z
#%%
def OR_OR (A,B,C,D):
    """realisation of OR-OR gate
    
    Parameters
    ----------
    A : list[]
    B : list[]
    C : list[]
    D : list[]

    Returns
    -------
    x : list[]
        intermediate output of A OR B
    y : list[]
        intermediate output of C OR D
    z : list[]
        Final output of x OR y

    """
    z=[]
    x=OR(A,B)
    y=OR(C,D)
    z=OR(x,y)
    return x,y,z
#%%
def OR_NAND (A,B,C,D):
    """realisation of OR-NAND gate
    
    Parameters
    ----------
    A : list[]
    B : list[]
    C : list[]
    D : list[]

    Returns
    -------
    x : list[]
        intermediate output of A OR B
    y : list[]
        intermediate output of C OR D
    z : list[]
        Final output of x NAND y

    """
    z=[]
    x=OR(A,B)
    y=OR(C,D)
    z=NAND(x,y)
    return x,y,z
#%%
def OR_NOR (A,B,C,D):
    """realisation of OR-NOR gate
    
    Parameters
    ----------
    A : list[]
    B : list[]
    C : list[]
    D : list[]

    Returns
    -------
    x : list[]
        intermediate output of A OR B
    y : list[]
        intermediate output of C OR D
    z : list[]
        Final output of x NOR y

    """
    z=[]
    x=OR(A,B)
    y=OR(C,D)
    z=NOR(x,y)
    return x,y,z
#%%
def NAND_AND (A,B,C,D):
    """realisation of NAND-AND gate
    
    Parameters
    ----------
    A : list[]
    B : list[]
    C : list[]
    D : list[]

    Returns
    -------
    x : list[]
        intermediate output of A NAND B
    y : list[]
        intermediate output of C NAND D
    z : list[]
        Final output of x AND y

    """
    z=[]
    x=NAND(A,B)
    y=NAND(C,D)
    z=AND(x,y)
    return x,y,z
#%%
def NAND_OR (A,B,C,D):
    """realisation of NAND-OR gate
    
    Parameters
    ----------
    A : list[]
    B : list[]
    C : list[]
    D : list[]

    Returns
    -------
    x : list[]
        intermediate output of A NAND B
    y : list[]
        intermediate output of C NAND D
    z : list[]
        Final output of x OR y

    """
    z=[]
    x=NAND(A,B)
    y=NAND(C,D)
    z=OR(x,y)
    return x,y,z
#%%
def NAND_NAND (A,B,C,D):
    """realisation of NAND-NAND gate
    
    Parameters
    ----------
    A : list[]
    B : list[]
    C : list[]
    D : list[]

    Returns
    -------
    x : list[]
        intermediate output of A NAND B
    y : list[]
        intermediate output of C NAND D
    z : list[]
        Final output of x NAND y

    """
    z=[]
    x=NAND(A,B)
    y=NAND(C,D)
    z=NAND(x,y)
    return x,y,z
#%%
def NAND_NOR (A,B,C,D):
    """realisation of NAND-NOR gate
    
    Parameters
    ----------
    A : list[]
    B : list[]
    C : list[]
    D : list[]

    Returns
    -------
    x : list[]
        intermediate output of A NAND B
    y : list[]
        intermediate output of C NAND D
    z : list[]
        Final output of x NOR y

    """
    z=[]
    x=NAND(A,B)
    y=NAND(C,D)
    z=NOR(x,y)
    return x,y,z
#%%
def NOR_AND (A,B,C,D):
    """realisation of NOR-AND gate
    
    Parameters
    ----------
    A : list[]
    B : list[]
    C : list[]
    D : list[]

    Returns
    -------
    x : list[]
        intermediate output of A NOR B
    y : list[]
        intermediate output of C NOR D
    z : list[]
        Final output of x AND y

    """
    z=[]
    x=NOR(A,B)
    y=NOR(C,D)
    z=AND(x,y)
    return x,y,z
#%%
def NOR_OR (A,B,C,D):
    """realisation of NOR-OR gate
    
    Parameters
    ----------
    A : list[]
    B : list[]
    C : list[]
    D : list[]

    Returns
    -------
    x : list[]
        intermediate output of A NOR B
    y : list[]
        intermediate output of C NOR D
    z : list[]
        Final output of x OR y

    """
    z=[]
    x=NOR(A,B)
    y=NOR(C,D)
    z=OR(x,y)
    return x,y,z
#%%
def NOR_NAND (A,B,C,D):
    """realisation of NOR-NAND gate
    
    Parameters
    ----------
    A : list[]
    B : list[]
    C : list[]
    D : list[]

    Returns
    -------
    x : list[]
        intermediate output of A NOR B
    y : list[]
        intermediate output of C NOR D
    z : list[]
        Final output of x NAND y

    """
    z=[]
    x=NOR(A,B)
    y=NOR(C,D)
    z=NAND(x,y)
    return x,y,z
#%%
def NOR_NOR (A,B,C,D):
    """realisation of NOR-NOR gate
    
    Parameters
    ----------
    A : list[]
    B : list[]
    C : list[]
    D : list[]

    Returns
    -------
    x : list[]
        intermediate output of A NOR B
    y : list[]
        intermediate output of C NOR D
    z : list[]
        Final output of x NOR y

    """
    z=[]
    x=NOR(A,B)
    y=NOR(C,D)
    z=NOR(x,y)
    return x,y,z
#%%


















# -*- coding: utf-8 -*-
"""
Created on Sun Jun 13 11:01:59 2021

@author: Vishwesh
"""
from Logic_Py.basic_gates import AND, OR, NOT, NAND, NOR,XNOR,XOR

#%%
def half_adder(A,B):
    """Half Adder realisation

    Parameters
    ----------
    A : list[]
    B : list[]

    Returns
    -------
    S : list[]
        Sum of two inputs
    C : list[]
        Carry of two inputs 

    """
    S = XOR (A,B)
    C = AND(A,B)
    return S,C
#%%
def full_adder(A,B,Cin):
    """Full Adder realisation - 3 bit Binary input

    Parameters
    ----------
    A : list[]
    B : list[]
    Cin : list[]
        Carry input bit

    Returns
    -------
    S : list[]
        sum of three inputs
    Cout : list[]
        carry of three inputs

    """
    S_1,C_1 = half_adder(A,B)
    S,C_2 = half_adder(Cin,S_1)
    Cout = OR(C_2,C_1)
    return S,Cout

#%%
























# -*- coding: utf-8 -*-
"""
Created on Thu Jun 10 17:34:19 2021

@author: Vishwesh
"""
from Logic_Py.basic_gates import AND, OR, NOT, NAND, NOR,XNOR,XOR
from Logic_Py.secondary_gates import AND_AND, AND_OR, AND_NAND, AND_NOR, OR_AND, OR_OR 
from Logic_Py.secondary_gates import OR_NAND, OR_NOR, NAND_AND, NAND_OR, NAND_NAND, NAND_NOR 
from Logic_Py.secondary_gates import NOR_AND, NOR_OR, NOR_NAND, NOR_NOR
from Logic_Py.arithmatic_gates import half_adder,full_adder
from Logic_Py.combinational_gates import Binary2Gray, Gray2Binary,EParity_gen,EParity_check,OParity_gen,OParity_check
from Logic_Py.plotting import plot_full_adder, plot_half_adder, plot_secondary, plot_basic



# -*- coding: utf-8 -*-
"""
Created on Wed Jun  9 15:00:29 2021

@author: Vishwesh
"""
import matplotlib.pyplot as plt
from Logic_Py.basic_gates import AND, OR, NOT, NAND, NOR,XNOR,XOR
from Logic_Py.secondary_gates import AND_AND, AND_OR, AND_NAND, AND_NOR, OR_AND, OR_OR 
from Logic_Py.secondary_gates import OR_NAND, OR_NOR, NAND_AND, NAND_OR, NAND_NAND, NAND_NOR 
from Logic_Py.secondary_gates import NOR_AND, NOR_OR, NOR_NAND, NOR_NOR
from Logic_Py.arithmatic_gates import half_adder,full_adder

#%% plots for basic gates

def plot_basic(x,y,gate):
    g=gate (x,y)
    plt.figure(1)
    plt.subplot(3, 1, 1)
    plt.xlim(0,len(x))
    plt.ylim(0,2)
    plt.ylabel("input 1")
    plt.grid(color = 'green', linestyle = '--', linewidth = 0.1,axis='x')
    plt.plot(x)
    plt.subplot(3, 1, 2)
    plt.xlim(0,len(y))
    plt.ylim(0,2)
    plt.ylabel("input 2")
    plt.grid(color = 'green', linestyle = '--', linewidth = 0.1,axis='x')
    plt.plot(y)
    plt.subplot(3, 1, 3)
    plt.xlim(0,len(g))
    plt.ylim(0,2)
    plt.xlabel("Instances")
    plt.ylabel("output")
    plt.plot(g,color='r')
    plt.grid(color = 'green', linestyle = '--', linewidth = 0.1,axis='x')
    plt.show()
    return g
    
#%% plots for secondary gates

def plot_secondary(x,y,z,q,gate1,gate2):
    g=gate1 (x,y)
    h=gate1 (z,q)
    i=gate2 (g,h)
    
    plt.figure(2)
    plt.subplot(3, 1, 1)
    plt.xlim(0,len(x))
    plt.ylim(0,2)
    plt.ylabel("gate 1")
    plt.grid(color = 'green', linestyle = '--', linewidth = 0.1,axis='x')
    plt.plot(g)
    plt.subplot(3, 1, 2)
    plt.xlim(0,len(y))
    plt.ylim(0,2)
    plt.ylabel("gate 2")
    plt.grid(color = 'green', linestyle = '--', linewidth = 0.1,axis='x')
    plt.plot(h)
    plt.subplot(3, 1, 3)
    plt.xlim(0,len(g))
    plt.ylim(0,2)
    plt.xlabel("Instances")
    plt.ylabel("output")
    plt.plot(g,color='r')
    plt.grid(color = 'green', linestyle = '--', linewidth = 0.1,axis='x')
    plt.show()
    return g,h,i

#%%
def plot_half_adder(A,B):
    s,c = half_adder(A,B)
    
    plt.figure(3)
    plt.subplot(2, 1, 1)
    plt.xlim(0,len(s))
    plt.ylim(0,2)
    plt.ylabel("Sum")
    plt.grid(color = 'green', linestyle = '--', linewidth = 0.1,axis='x')
    plt.plot(s)
    plt.subplot(2, 1, 2)
    plt.xlim(0,len(c))
    plt.ylim(0,2)
    plt.ylabel("Carry")
    plt.grid(color = 'green', linestyle = '--', linewidth = 0.1,axis='x')
    plt.xlabel("Instances")
    plt.plot(c,color='r')
    plt.show()
    return s,c    
    
#%%
def plot_full_adder(A,B,Cin):
    s,c = full_adder(A, B, Cin)
    
    plt.figure(3)
    plt.subplot(2, 1, 1)
    plt.xlim(0,len(s))
    plt.ylim(0,2)
    plt.ylabel("Sum")
    plt.grid(color = 'green', linestyle = '--', linewidth = 0.1,axis='x')
    plt.plot(s)
    plt.subplot(2, 1, 2)
    plt.xlim(0,len(c))
    plt.ylim(0,2)
    plt.ylabel("Carry")
    plt.grid(color = 'green', linestyle = '--', linewidth = 0.1,axis='x')
    plt.xlabel("Instances")
    plt.plot(c,color='r')
    plt.show()
    return s,c    

#%%






# -*- coding: utf-8 -*-

"""
Created on Sun Jun  6 15:21:25 2021

@author: Vishwesh
"""
import numpy as np
import scipy
import matplotlib.pyplot as plt
#%%
def OR (A,B):
    """realisation of AND gate
    
    Parameters
    ----------
    A : list[]
        0 or 1
    B : list[]
        0 or 1

    Returns
    -------
    result : list[]

    """
    result=[]
    for i in range(len(A)):
        
        if A[i] == 1 or B[i] == 1:
            result.append(1)
        else:
            result.append(0)
    return result

#%%
def AND (A,B):
    """realisation of AND gate

    Parameters
    ----------
    A : list[]
        0 or 1
    B : list[]
        0 or 1

    Returns
    -------
    result : list[]

    """
    result=[]
    for i in range(len(A)):
        
        if A[i] == 1 and B[i] == 1:
            result.append(1)
        else:
            result.append(0)
    return result

#%%
def NOT (A):
    """realisation of NOT gate

    Parameters
    ----------
    A : list[]
        0 or 1

    Returns
    -------
    result : list[]

    """
    result=[]
    for i in range(len(A)):
        
        if A[i] == 1:
            result.append(0)
        else:
            result.append(1)
    return result
                
#%%
def NAND (A,B):
    """realisation of NAND gate

    Parameters
    ----------
    A : list[]
        0 or 1
    B : list[]
        0 or 1

    Returns
    -------
    result : list[]
        
    """
    result=[]
    for i in range(len(A)):
        
        if A[i] == 1 and B[i] == 1:
            result.append(0)
        else:
            result.append(1)
    return result

#%%
def NOR (A,B):
    """realisation of NOR gate

    Parameters
    ----------
    A : list[]
        0 or 1
    B : list[]
        0 or 1

    Returns
    -------
    result : list[]
        
    """
    result=[]
    for i in range(len(A)):
        
        if A[i] == 1 or B[i] == 1:
            result.append(0)
        else:
            result.append(1)
    return result

#%% XNOR
def XNOR (A,B):
    """realisation of XNOR gate

    Parameters
    ----------
    A : list[]
        0 or 1
    B : list[]
        0 or 1

    Returns
    -------
    result : list[]
        
    """
    result=[]
    for i in range(len(A)):
        
        if A[i] == 1 and B[i] == 1:
            result.append(1)
        elif A[i] == 0 and B[i] == 0:
            result.append(1)    
        else:
            result.append(0)
    return result

#%% XOR
def XOR (A,B):
    """realisation of XOR gate

    Parameters
    ----------
    A : list[]
        0 or 1
    B : list[]
        0 or 1

    Returns
    -------
    result : list[]
        
    """
    result=[]
    for i in range(len(A)):
        
        if A[i] == 1 and B[i] == 1:
            result.append(0)
        elif A[i] == 0 and B[i] == 0:
            result.append(0)    
        else:
            result.append(1)
    return result

#%%




# -*- coding: utf-8 -*-
"""
Created on Sun Jun 13 17:48:10 2021

@author: Vishwesh
"""
from Logic_Py.basic_gates import AND
from Logic_Py.plotting import plot_basic

x = [1,0,0,1,0,1,1,0,1,0]
y = [0,1,0,1,0,1,0,1,0,0]
z = [1,1,0,1,1,0,0,1,0,0]

result = AND (x,y)
plot_basic(x, y, AND)

#%%
from Logic_Py.arithmatic_gates import full_adder
from Logic_Py.plotting import plot_full_adder

sum_,carry_ = full_adder(x, y, z)
plot_full_adder(x,y,z)

#%%



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['logic_py']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'logic-py',
    'version': '0.3.4.1',
    'description': 'Python Package to realize combinational logic gates',
    'long_description': '# Logic_Py\n\n[![Made with Python3](https://img.shields.io/badge/Made%20With-Python3-green)](https://www.python.org/)\n[![GitHub license](https://img.shields.io/badge/license-MIT-blue.svg)](https://github.com/vishwesh-vishwesh/Logic_Py/blob/main/LICENSE)\n[![Github version](https://img.shields.io/badge/version-0.3.4-green)](https://github.com/vishwesh-vishwesh/Logic_Py)\n\nThis Python package enables the user to realise Logic based combinational circuits built on basic logic gates.\nAll the inputs must be binary and of same length for the functions to perform desired operation. \n\n## Installation\n- pip install logic-py\n- from Logic_Py import AND, full_adder, plot_secondary etc.,\n\n## Basic Gates\n\nThere are 7 basic gates, all other secondaary and combinational gates are the combinations of these 7 basic gates.\n- AND, OR, NOT, NAND, NOR,XNOR,XOR\n- ex : from Logic_Py import AND\n\n## Secondary Gates\n\nThere are 16 Secondary gates, which take 4 binary inputs and 1 binary output.\n- AND_AND, AND_OR, AND_NAND, AND_NOR, OR_AND, OR_OR, \n   OR_NAND, OR_NOR, NAND_AND, NAND_OR, NAND_NAND, \n   NAND_NOR, NOR_AND, NOR_OR, NOR_NAND, NOR_NOR,\n\n## Combinational Gates\nFew combinational circuits are added as start in this beta version, few more will follow in the coming update.\n- Binary2Gray, Gray2Binary, EParity_gen, EParity_check, OParity_gen, OParity_check\n\n## Arithmatic Gates\nTwo arithmatic gates are added for the beta version, more will follow in the coming update.\n- Half Adder\n- Full Adder\n\n## Plots\nPlots for the basic gates, secondary gates and arithmatic gates are available with the current version.\n- plot_full_adder, plot_half_adder, plot_secondary, plot_basic\n\n## Citation\n- [Tutorialspoint - digital circuit basics](https://www.tutorialspoint.com/digital_circuits)\n\n>Use [Github](https://github.com/vishwesh-vishwesh/Logic_Py/) for further updates. \n>Please kindly cite incase you use the package and fork.\n\n>Use Hellow world example for the syntax\n>or use help function in python console\n>ex: help(AND)\n\n',
    'author': 'Vishwesh',
    'author_email': 'vishwesh.arush@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Vishwesh-Vishwesh/Logic_Py',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



# -*- coding: utf-8 -*-
"""
Created on Sat Jun 12 10:24:51 2021

@author: Vishwesh
"""

from Logic_Py.basic_gates import AND, OR, NOT, NAND, NOR,XNOR,XOR
#%%
def Binary2Gray(A,B,C,D):
    """4 bit Binary to Gray converter
    
    Parameters - input Binary code
    ----------
    A : list[]
    B : list[]
    C : list[]
    D : list[]

    Returns - outputs the Gray code
    -------
    w : list[]
    x : list[]
    y : list[]
    z : list[]

    """
    w = A
    x = XOR (A,B)
    y = XOR (B,C)
    z = XOR (C,D)
    return w,x,y,z
#%%
def Gray2Binary(A,B,C,D):
    """4 bit Gray to Binary converter
    
    Parameters - input gray code
    ----------
    A : list[]
    B : list[]
    C : list[]
    D : list[]

    Returns - outputs the binary code
    -------
    w : list[]
    x : list[]
    y : list[]
    z : list[]

    """
    w = A
    x = XOR (A,B)
    y = XOR (x,C)
    z = XOR (y,D)
    return w,x,y,z
#%% 
def EParity_gen(A,B,C):
    """3 bit Even Parity generator for 3 bit Binary input
    
    Parameters
    ----------
    A : list[]
    B : list[]
    C : list[]

    Returns
    -------
    parity : list[]
        generated even parity bit

    """
    z = XOR (A,B)
    parity = XOR (z,C)
    return parity
#%%

def OParity_gen(A,B,C):
    """3 bit Even Parity generator for 3 bit Binary input
    
    Parameters
    ----------
    A : list[]
    B : list[]
    C : list[]

    Returns
    -------
    parity : list[]
        generated even parity bit

    """
    z = XOR (A,B)
    parity = XNOR (z,C)
    return parity

#%%

def EParity_check(A,B,C,P):
    """Even Parity Checker for 3 bit binary input with a parity bit P (4-bit input in total)
    
    Parameters
    ----------
    A : list[]
    B : list[]
    C : list[]
    P : list[]
        Even parity bit as 4th input
        (Condition : Program outputs a value even if the P value is not an Even parity bit,
        it is prerogative of user use the correct P value for input)

    Returns
    -------
    E : list[]
        outputs an Even parity check bit, E[i] will be 1 if the input at i has odd number of 1s.

    """
    x = XOR (A,B)
    y = XOR (C,P)
    E = XOR (x,y)
    return E

#%%

def OParity_check(A,B,C,P):
    """Odd Parity Checker for 3 bit binary input with a parity bit P (4-bit input in total)
    
    Parameters
    ----------
    A : list[]
    B : list[]
    C : list[]
    P : list[]
        Odd parity bit as 4th input
        (Condition : Program outputs a value even if the P value is not an appropriate Odd parity bit,
        it is prerogative of user to use the correct P value for input)

    Returns - Odd parity check bit
    -------
    O : list[]
        outputs an Odd parity check bit, O[i] will be 1 if the input at i has even number of 1s.
    """    
    x = XOR (A,B)
    y = XOR (C,P)
    O = XNOR (x,y)
    return O
#%%    









